package com.pixelai;

import com.base.interceptor.ContextInjectInterceptor;
import io.swagger.annotations.Api;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.context.annotation.Bean;
import org.springframework.http.ResponseEntity;
import org.springframework.scheduling.annotation.EnableAsync;
import org.springframework.scheduling.annotation.EnableScheduling;
import springfox.documentation.builders.ApiInfoBuilder;
import springfox.documentation.builders.ParameterBuilder;
import springfox.documentation.builders.PathSelectors;
import springfox.documentation.builders.RequestHandlerSelectors;
import springfox.documentation.schema.ModelRef;
import springfox.documentation.service.ApiInfo;
import springfox.documentation.service.Parameter;
import springfox.documentation.spi.DocumentationType;
import springfox.documentation.spring.web.plugins.Docket;
import springfox.documentation.swagger2.annotations.EnableSwagger2;

import java.time.LocalDate;
import java.util.ArrayList;
import java.util.List;
@EnableAsync
@EnableScheduling
@SpringBootApplication
@EnableSwagger2
public class PixelaiApplication {

    public static void main(String[] args) {
        SpringApplication.run(PixelaiApplication.class, args);
    }

    /**
     * 生成API文档的入口
     */
    @Bean
    public Docket generateApi() {
        ParameterBuilder tokenPar = new ParameterBuilder();
        List<Parameter>  pars     = new ArrayList<Parameter>();
        tokenPar.name(ContextInjectInterceptor.TOKEN).description("安全令牌").modelRef(new ModelRef("string")).parameterType("header").defaultValue(ContextInjectInterceptor.SWAGGER_TOKEN).required(false).build();
        pars.add(tokenPar.build());
        return new Docket(DocumentationType.SWAGGER_2).apiInfo(this.apiInfo()).select()
                .apis(RequestHandlerSelectors.withClassAnnotation(Api.class)).paths(PathSelectors.any()).build().globalOperationParameters(pars)
                .pathMapping("/").directModelSubstitute(LocalDate.class, String.class)
                .genericModelSubstitutes(ResponseEntity.class).useDefaultResponseMessages(false);
    }

    private ApiInfo apiInfo() {
        return new ApiInfoBuilder().title("Pixelai API").build();
    }

}
