package com.pixelai.api.base.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.base.annotation.SysLog;
import com.base.helper.BaseController;
import com.base.helper.Result;
import com.pixelai.api.base.entity.AcPage;
import com.pixelai.api.base.service.AcPageService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.Collections;
import java.util.List;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author mzl
 * @since 2020-07-09
 */
@Api(tags = "页面")
@RestController
@RequestMapping("/acPage")
public class AcPageController extends BaseController {

    private static final Logger        LOG = LoggerFactory.getLogger(AcPageController.class);
    @Autowired
    private              AcPageService entityService;

    @SysLog(action = "acPage:findByPage", value = "页面管理：页面列表")
    @ApiOperation(value = "分页查询页面", notes = "分页查询页面")
    @RequestMapping(method = RequestMethod.GET)
    @ApiImplicitParams({@ApiImplicitParam(name = "size", value = "分页大小", paramType = "query"),
            @ApiImplicitParam(name = "current", value = "当前页面：从1开始", paramType = "query")})
    public Result<IPage<AcPage>> findByPage(final AcPage example, final Page page) {
        IPage<AcPage> records = entityService.page(example, page);
        return new Result(true, records);
    }

    @SysLog(action = "acPage:delete", value = "页面管理：删除")
    @ApiOperation(value = "删除页面")
    @RequestMapping(value = "{id}", method = {RequestMethod.DELETE})
    @ApiImplicitParam(name = "id", value = "页面ID", required = true, paramType = "path")
    public Result delete(@PathVariable final Integer id) {
        entityService.removeById(id);
        return new Result(true, "成功删除页面", null);
    }

    @SysLog(action = "acPage:one", value = "页面管理：查询")
    @ApiOperation(value = "查询单个页面")
    @RequestMapping(value = "{id}", method = {RequestMethod.GET})
    @ApiImplicitParam(name = "id", value = "页面ID", required = true, paramType = "path")
    public Result one(@PathVariable final Integer id) {
        AcPage entity = entityService.getById(id);
        return new Result(true, entity);
    }

    @SysLog(action = "acPage:add", value = "页面管理：添加")
    @ApiOperation(value = "添加页面")
    @RequestMapping(method = {RequestMethod.POST})
    public Result add(@Valid @RequestBody final AcPage entity) {
        entityService.save(entity);
        return new Result(true, "成功保存页面", null);
    }

    @SysLog(action = "acPage:update", value = "页面管理：修改")
    @ApiOperation(value = "修改页面")
    @RequestMapping(method = {RequestMethod.PUT})
    public Result update(@Valid @RequestBody final AcPage entity) {
        if (null == entity.getId()) {
            throw new RuntimeException("id不能为空");
        }
        entityService.updateById(entity);
        return new Result(true, "成功修改页面", null);
    }

    @SysLog(action = "acPage:all", value = "页面管理：全部页面")
    @ApiOperation(value = "全部页面")
    @RequestMapping(value = "all", method = RequestMethod.GET)
    public Result<List<AcPage>> all(AcPage example) {
        List<AcPage> entitys = entityService.list(example);
        if (null != entitys) {
            return new Result<>(true, entitys);
        }
        return new Result<>(true, Collections.emptyList());
    }

}

