package com.pixelai.api.component.controller;

import org.springframework.web.bind.annotation.RequestMapping;
import com.base.annotation.SysLog;
import org.springframework.web.bind.annotation.RestController;
import com.base.helper.BaseController;
import com.base.helper.Result;
import com.pixelai.api.component.entity.CpText;
import com.pixelai.api.component.service.CpTextService;
import io.swagger.annotations.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DataIntegrityViolationException;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;

import javax.validation.Valid;
import java.util.Collections;
import java.util.List;
import java.util.Map;

/**
 * <p>
 *  前端控制器
 * </p>
 *
 * @author gjj
 * @since 2024-10-26
 */
@Api(tags = "文章内容管理")
@RestController
@RequestMapping("/cpText")
public class CpTextController extends BaseController {

        private static final Logger       LOG = LoggerFactory.getLogger(CpTextController.class);
        @Autowired
        private              CpTextService entityService;

        @SysLog(action = "findByPage", value = "分页查询文章内容")
        @ApiOperation(value = "分页查询文章内容", notes = "分页查询文章内容")
        @RequestMapping(method = RequestMethod.GET)
        @ApiImplicitParams({@ApiImplicitParam(name = "size", value = "分页大小", paramType = "query"),
                @ApiImplicitParam(name = "current", value = "当前页面：从1开始", paramType = "query")})
        public Result<IPage<CpText>> findByPage(final CpText example, final Page page) {
            IPage<CpText>        records    = entityService.page(example,page);
            return new Result(true, records);
        }

        @SysLog(action = "delete", value = "删除文章内容")
        @ApiOperation(value = "删除文章内容")
        @RequestMapping(value = "{id}", method = {RequestMethod.DELETE})
        @ApiImplicitParam(name = "id", value = "文章内容ID", required = true, paramType = "path")
        public Result delete(@PathVariable final Integer id) {
            try {
                entityService.removeById(id);
                return new Result(true, "成功删除文章内容", null);
            } catch (DataIntegrityViolationException e) {
                LOG.error("删除文章内容失败", e);
                return new Result(false, "删除文章内容失败", "该文章内容不能删除，存在其他关联数据");
            } catch (Exception e) {
                LOG.error("删除文章内容失败", e);
                return new Result(false, "删除文章内容失败", e.getMessage());
            }
        }

        @SysLog(action = "one", value = "查询单个文章内容")
        @ApiOperation(value = "查询单个文章内容")
        @RequestMapping(value = "{id}", method = {RequestMethod.GET})
        @ApiImplicitParam(name = "id", value = "文章内容ID", required = true, paramType = "path")
        public Result one(@PathVariable final Integer id) {
            try {
                CpText entity = entityService.getById(id);
                return new Result(true, entity);
            } catch (Exception e) {
                LOG.error("查询单个文章内容失败", e);
                return new Result(false, new CpText());
            }
        }

        @SysLog(action = "add", value = "添加文章内容")
        @ApiOperation(value = "添加文章内容")
        @RequestMapping(method = {RequestMethod.POST})
        public Result add(@Valid @RequestBody final CpText entity, final BindingResult result) {
            try {
                if (result.hasErrors()) {
                    Map<String, String> map      = this.getErrors(result);
                    String              errorMsg = map.entrySet().iterator().next().getValue();
                    return new Result(false, "保存文章内容失败", errorMsg, map);
                } else {
                    entityService.save(entity);
                    return new Result(true, "成功保存文章内容", null);
                }
            } catch (Exception e) {
                LOG.error("添加文章内容失败", e);
                return new Result(false, "保存文章内容失败", e.getMessage());
            }
        }

        @SysLog(action = "update", value = "修改文章内容")
        @ApiOperation(value = "修改文章内容")
        @RequestMapping(method = {RequestMethod.PUT})
        public Result update(@Valid @RequestBody final CpText entity, final BindingResult result) {
            try {
                if (result.hasErrors()) {
                    Map<String, String> map      = this.getErrors(result);
                    String              errorMsg = map.entrySet().iterator().next().getValue();
                    return new Result(false, "修改文章内容失败", errorMsg, map);
                } else {
                    if (null == entity.getId()) {
                         throw new RuntimeException("id不能为空");
                    }
                    entityService.updateById(entity);
                    return new Result(true, "成功修改文章内容", null);
                }
            } catch (Exception e) {
                LOG.error("修改文章内容失败", e);
                return new Result(false, "修改文章内容失败", e.getMessage());
            }
        }

        @ApiOperation(value = "全部文章内容")
        @RequestMapping(value = "all", method = RequestMethod.GET)
        public Result<List<CpText>> all(CpText example) {
            List<CpText> entitys = entityService.list(example);
            if (null != entitys) {
                return new Result<>(true, entitys);
            }
            return new Result<>(true, Collections.emptyList());
        }

}

