package com.pixelai.api.group.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.base.annotation.SysLog;
import com.base.helper.BaseController;
import com.base.helper.Result;
import com.pixelai.api.group.entity.AcAction;
import com.pixelai.api.group.service.AcActionService;
import com.pixelai.api.group.vo.AcActionVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.Collections;
import java.util.Date;
import java.util.List;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author mzl
 * @since 2020-07-09
 */
@Api(tags = "功能")
@RestController
@RequestMapping("/acAction")
public class AcActionController extends BaseController {

    private static final Logger          LOG = LoggerFactory.getLogger(AcActionController.class);
    @Autowired
    private              AcActionService entityService;

    @SysLog(action = "acAction:findByPage", value = "角色管理：功能列表")
    @ApiOperation(value = "分页查询功能", notes = "分页查询功能")
    @RequestMapping(method = RequestMethod.GET)
    @ApiImplicitParams({@ApiImplicitParam(name = "size", value = "分页大小", paramType = "query"),
            @ApiImplicitParam(name = "current", value = "当前页面：从1开始", paramType = "query")})
    public Result<IPage<AcActionVo>> findByPage(final AcActionVo example, final Page page) {
        IPage<AcActionVo> records = entityService.findByPage(example, page);
        return new Result(true, records);
    }

    @SysLog(action = "acAction:delete", value = "角色管理：删除功能")
    @ApiOperation(value = "删除功能")
    @RequestMapping(value = "{id}", method = {RequestMethod.DELETE})
    @ApiImplicitParam(name = "id", value = "功能ID", required = true, paramType = "path")
    public Result delete(@PathVariable final Integer id) {
        AcAction acAction = entityService.getById(id);
        if (null == acAction) {
            throw new RuntimeException("没有找到id对应的对象");
        }
        acAction.setState("0");
        entityService.updateById(acAction);
        return new Result(true, "成功删除功能", null);
    }

    @SysLog(action = "acAction:one", value = "角色管理：查询功能详情")
    @ApiOperation(value = "查询单个功能")
    @RequestMapping(value = "{id}", method = {RequestMethod.GET})
    @ApiImplicitParam(name = "id", value = "功能ID", required = true, paramType = "path")
    public Result one(@PathVariable final Integer id) {
        AcAction entity = entityService.getById(id);
        return new Result(true, entity);
    }

    @SysLog(action = "acAction:add", value = "角色管理：添加功能")
    @ApiOperation(value = "添加功能")
    @RequestMapping(method = {RequestMethod.POST})
    public Result add(@Valid @RequestBody final AcAction entity) {
        AcAction old = entityService.findByAcAction(entity.getAction());
        if (null != old) {
            throw new RuntimeException("已经存在相同编码的功能");
        }
        if (null == entity.getId()) {
            entity.setCreatetime(new Date());
            entity.setState("1");
        }
        entityService.save(entity);
        return new Result(true, "成功保存功能", null);
    }

    @SysLog(action = "acAction:update", value = "角色管理：修改功能")
    @ApiOperation(value = "修改功能")
    @RequestMapping(method = {RequestMethod.PUT})
    public Result update(@Valid @RequestBody final AcAction entity) {
        if (null == entity.getId()) {
            throw new RuntimeException("id不能为空");
        }
        entityService.updateById(entity);
        return new Result(true, "成功修改功能", null);
    }

    @SysLog(action = "acAction:all", value = "角色管理：全部功能")
    @ApiOperation(value = "全部功能")
    @RequestMapping(value = "all", method = RequestMethod.GET)
    public Result<List<AcActionVo>> all(AcActionVo example) {
        List<AcActionVo> entitys = entityService.findByExample(example);
        if (null != entitys) {
            return new Result<>(true, entitys);
        }
        return new Result<>(true, Collections.emptyList());
    }

}

