package com.pixelai.api.group.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.base.annotation.SysLog;
import com.base.helper.BaseController;
import com.base.helper.Result;
import com.pixelai.api.group.entity.AcGroup;
import com.pixelai.api.group.service.AcGroupService;
import com.pixelai.api.group.vo.AcGroupVo;
import com.pixelai.api.user.service.AcUserService;
import com.pixelai.api.user.vo.AcUserVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.Collections;
import java.util.Date;
import java.util.List;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author mzl
 * @since 2020-07-09
 */
@Api(tags = "角色")
@RestController
@RequestMapping("/acGroup")
public class AcGroupController extends BaseController {

    private static final Logger         LOG = LoggerFactory.getLogger(AcGroupController.class);
    @Autowired
    private              AcGroupService entityService;
    @Autowired
    private              AcUserService  acUserService;

    @SysLog(action = "acGroup:findByPage", value = "角色管理：角色列表")
    @ApiOperation(value = "分页查询角色", notes = "分页查询角色")
    @RequestMapping(method = RequestMethod.GET)
    @ApiImplicitParams({@ApiImplicitParam(name = "size", value = "分页大小", paramType = "query"),
            @ApiImplicitParam(name = "current", value = "当前页面：从1开始", paramType = "query")})
    public Result<IPage<AcGroupVo>> findByPage(final AcGroupVo example, final Page page) {
        example.setGrouptypeid(1);
        IPage<AcGroupVo> records = entityService.findByPage(example, page);
        return new Result(true, records);
    }

    @SysLog(action = "acGroup:delete", value = "角色管理：删除角色")
    @ApiOperation(value = "删除角色")
    @RequestMapping(value = "{id}", method = {RequestMethod.DELETE})
    @ApiImplicitParam(name = "id", value = "角色ID", required = true, paramType = "path")
    public Result delete(@PathVariable final Integer id) {
        AcGroup acGroup = entityService.getById(id);
        if (null == acGroup) {
            throw new RuntimeException("没有找到id对应对象");
        }
        List<AcUserVo> acUsers = acUserService.findByGroupid(acGroup.getId());
        if (null != acUsers && acUsers.size() > 0) {
            throw new RuntimeException("存在该角色的订单");
        }
        acGroup.setState("0");
        entityService.updateById(acGroup);
        return new Result(true, "成功删除角色", null);
    }

    @SysLog(action = "acGroup:one", value = "角色管理：查询角色详情")
    @ApiOperation(value = "查询单个角色")
    @RequestMapping(value = "{id}", method = {RequestMethod.GET})
    @ApiImplicitParam(name = "id", value = "角色ID", required = true, paramType = "path")
    public Result one(@PathVariable final Integer id) {
        AcGroup entity = entityService.getById(id);
        return new Result(true, entity);
    }

    @SysLog(action = "acGroup:add", value = "角色管理：添加角色")
    @ApiOperation(value = "添加角色")
    @RequestMapping(method = {RequestMethod.POST})
    public Result add(@Valid @RequestBody final AcGroup entity) {
        AcGroup old = entityService.findByAcgroup(entity.getAcgroup());
        if (null != old) {
            throw new RuntimeException("已经存在相同编码的角色");
        }
        if (null == entity.getId()) {
            entity.setGrouptypeid(1);
            entity.setShow("1");
            entity.setEdit("2");
            entity.setCreatetime(new Date());
            entity.setState("1");
        }
        entityService.save(entity);
        return new Result(true, "成功保存角色", null);
    }

    @SysLog(action = "acGroup:update", value = "角色管理：修改角色")
    @ApiOperation(value = "修改角色")
    @RequestMapping(method = {RequestMethod.PUT})
    public Result update(@Valid @RequestBody final AcGroup entity) {
        if (null == entity.getId()) {
            throw new RuntimeException("id不能为空");
        }
        entityService.updateById(entity);
        return new Result(true, "成功修改角色", null);
    }

    @SysLog(action = "acGroup:all", value = "角色管理：全部角色")
    @ApiOperation(value = "全部角色")
    @RequestMapping(value = "all", method = RequestMethod.GET)
    public Result<List<AcGroupVo>> all(AcGroupVo example) {
        List<AcGroupVo> entitys = entityService.findByExample(example);
        if (null != entitys) {
            return new Result<>(true, entitys);
        }
        return new Result<>(true, Collections.emptyList());
    }

}

