package com.pixelai.api.pa.controller;

import org.springframework.web.bind.annotation.RequestMapping;
import com.base.annotation.SysLog;
import org.springframework.web.bind.annotation.RestController;
import com.base.helper.BaseController;
import com.base.helper.Result;
import com.pixelai.api.pa.entity.PaConsumption;
import com.pixelai.api.pa.service.PaConsumptionService;
import io.swagger.annotations.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DataIntegrityViolationException;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;

import javax.validation.Valid;
import java.util.Collections;
import java.util.List;
import java.util.Map;

/**
 * <p>
 *  前端控制器
 * </p>
 *
 * @author gjj
 * @since 2024-12-04
 */
@Api(tags = "PaConsumption")
@RestController
@RequestMapping("/paConsumption")
public class PaConsumptionController extends BaseController {

        private static final Logger       LOG = LoggerFactory.getLogger(PaConsumptionController.class);
        @Autowired
        private              PaConsumptionService entityService;

        @SysLog(action = "findByPage", value = "分页查询订单")
        @ApiOperation(value = "分页查询订单", notes = "分页查询订单")
        @RequestMapping(method = RequestMethod.GET)
        @ApiImplicitParams({@ApiImplicitParam(name = "size", value = "分页大小", paramType = "query"),
                @ApiImplicitParam(name = "current", value = "当前页面：从1开始", paramType = "query")})
        public Result<IPage<PaConsumption>> findByPage(final PaConsumption example, final Page page) {
            IPage<PaConsumption>        records    = entityService.page(example,page);
            return new Result(true, records);
        }

        @SysLog(action = "delete", value = "删除订单")
        @ApiOperation(value = "删除订单")
        @RequestMapping(value = "{id}", method = {RequestMethod.DELETE})
        @ApiImplicitParam(name = "id", value = "订单ID", required = true, paramType = "path")
        public Result delete(@PathVariable final Integer id) {
            try {
                entityService.removeById(id);
                return new Result(true, "成功删除订单", null);
            } catch (DataIntegrityViolationException e) {
                LOG.error("删除订单失败", e);
                return new Result(false, "删除订单失败", "该订单不能删除，存在其他关联数据");
            } catch (Exception e) {
                LOG.error("删除订单失败", e);
                return new Result(false, "删除订单失败", e.getMessage());
            }
        }

        @SysLog(action = "one", value = "查询单个订单")
        @ApiOperation(value = "查询单个订单")
        @RequestMapping(value = "{id}", method = {RequestMethod.GET})
        @ApiImplicitParam(name = "id", value = "订单ID", required = true, paramType = "path")
        public Result one(@PathVariable final Integer id) {
            try {
                PaConsumption entity = entityService.getById(id);
                return new Result(true, entity);
            } catch (Exception e) {
                LOG.error("查询单个订单失败", e);
                return new Result(false, new PaConsumption());
            }
        }

        @SysLog(action = "add", value = "添加订单")
        @ApiOperation(value = "添加订单")
        @RequestMapping(method = {RequestMethod.POST})
        public Result add(@Valid @RequestBody final PaConsumption entity, final BindingResult result) {
            try {
                if (result.hasErrors()) {
                    Map<String, String> map      = this.getErrors(result);
                    String              errorMsg = map.entrySet().iterator().next().getValue();
                    return new Result(false, "保存订单失败", errorMsg, map);
                } else {
                    entityService.save(entity);
                    return new Result(true, "成功保存订单", null);
                }
            } catch (Exception e) {
                LOG.error("添加订单失败", e);
                return new Result(false, "保存订单失败", e.getMessage());
            }
        }

        @SysLog(action = "update", value = "修改订单")
        @ApiOperation(value = "修改订单")
        @RequestMapping(method = {RequestMethod.PUT})
        public Result update(@Valid @RequestBody final PaConsumption entity, final BindingResult result) {
            try {
                if (result.hasErrors()) {
                    Map<String, String> map      = this.getErrors(result);
                    String              errorMsg = map.entrySet().iterator().next().getValue();
                    return new Result(false, "修改订单失败", errorMsg, map);
                } else {
                    if (null == entity.getId()) {
                         throw new RuntimeException("id不能为空");
                    }
                    entityService.updateById(entity);
                    return new Result(true, "成功修改订单", null);
                }
            } catch (Exception e) {
                LOG.error("修改订单失败", e);
                return new Result(false, "修改订单失败", e.getMessage());
            }
        }

        @ApiOperation(value = "全部订单")
        @RequestMapping(value = "all", method = RequestMethod.GET)
        public Result<List<PaConsumption>> all(PaConsumption example) {
            List<PaConsumption> entitys = entityService.list(example);
            if (null != entitys) {
                return new Result<>(true, entitys);
            }
            return new Result<>(true, Collections.emptyList());
        }

}

