package com.pixelai.api.pa.controller;

import com.pixelai.api.pa.dao.PaCreationMapper;
import com.pixelai.api.pa.vo.PaCreationVo;
import org.springframework.beans.BeanUtils;
import org.springframework.web.bind.annotation.RequestMapping;
import com.base.annotation.SysLog;
import org.springframework.web.bind.annotation.RestController;
import com.base.helper.BaseController;
import com.base.helper.Result;
import com.pixelai.api.pa.entity.PaCreation;
import com.pixelai.api.pa.service.PaCreationService;
import io.swagger.annotations.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DataIntegrityViolationException;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;

import javax.validation.Valid;
import java.util.Collections;
import java.util.List;
import java.util.Map;

/**
 * <p>
 *  前端控制器
 * </p>
 *
 * @author gjj
 * @since 2024-12-05
 */
@Api(tags = "会员作品")
@RestController
@RequestMapping("/paCreation")
public class PaCreationController extends BaseController {

    private static final Logger       LOG = LoggerFactory.getLogger(PaCreationController.class);
    @Autowired
    private              PaCreationService entityService;
    @Autowired
    private PaCreationMapper paCreationMapper;

    @SysLog(action = "findByPage", value = "分页查询作品")
    @ApiOperation(value = "分页查询作品", notes = "分页查询作品")
    @RequestMapping(method = RequestMethod.GET)
    @ApiImplicitParams({@ApiImplicitParam(name = "size", value = "分页大小", paramType = "query"),
            @ApiImplicitParam(name = "current", value = "当前页面：从1开始", paramType = "query")})
    public Result<IPage<PaCreationVo>> findByPage(final PaCreationVo example, final Page page) {
        IPage<PaCreationVo>        records    = entityService.findByPage(example,page);
        return new Result(true, records);
    }

    @SysLog(action = "delete", value = "删除作品")
    @ApiOperation(value = "删除作品")
    @RequestMapping(value = "{id}", method = {RequestMethod.DELETE})
    @ApiImplicitParam(name = "id", value = "作品ID", required = true, paramType = "path")
    public Result delete(@PathVariable final Integer id) {
        try {
            PaCreation paCreation = entityService.getById(id);
            paCreation.setState("f");
            entityService.updateById(paCreation);
            return new Result(true, "成功删除作品", null);
        } catch (DataIntegrityViolationException e) {
            LOG.error("删除作品失败", e);
            return new Result(false, "删除作品失败", "该作品不能删除，存在其他关联数据");
        } catch (Exception e) {
            LOG.error("删除作品失败", e);
            return new Result(false, "删除作品失败", e.getMessage());
        }
    }

    @SysLog(action = "one", value = "查询单个作品")
    @ApiOperation(value = "查询单个作品")
    @RequestMapping(value = "{id}", method = {RequestMethod.GET})
    @ApiImplicitParam(name = "id", value = "作品ID", required = true, paramType = "path")
    public Result one(@PathVariable final Integer id) {
        try {
            PaCreationVo entity = paCreationMapper.findById(id);
            return new Result(true, entity);
        } catch (Exception e) {
            LOG.error("查询单个作品失败", e);
            return new Result(false, new PaCreation());
        }
    }

//    @SysLog(action = "add", value = "添加作品")
//    @ApiOperation(value = "添加作品")
//    @RequestMapping(method = {RequestMethod.POST})
//    public Result add(@Valid @RequestBody final PaCreation entity, final BindingResult result) {
//        try {
//            if (result.hasErrors()) {
//                Map<String, String> map      = this.getErrors(result);
//                String              errorMsg = map.entrySet().iterator().next().getValue();
//                return new Result(false, "保存作品失败", errorMsg, map);
//            } else {
//                entityService.save(entity);
//                return new Result(true, "成功保存作品", null);
//            }
//        } catch (Exception e) {
//            LOG.error("添加作品失败", e);
//            return new Result(false, "保存作品失败", e.getMessage());
//        }
//    }
//
//    @SysLog(action = "update", value = "修改作品")
//    @ApiOperation(value = "修改作品")
//    @RequestMapping(method = {RequestMethod.PUT})
//    public Result update(@Valid @RequestBody final PaCreation entity, final BindingResult result) {
//        try {
//            if (result.hasErrors()) {
//                Map<String, String> map      = this.getErrors(result);
//                String              errorMsg = map.entrySet().iterator().next().getValue();
//                return new Result(false, "修改作品失败", errorMsg, map);
//            } else {
//                if (null == entity.getId()) {
//                    throw new RuntimeException("id不能为空");
//                }
//                entityService.updateById(entity);
//                return new Result(true, "成功修改作品", null);
//            }
//        } catch (Exception e) {
//            LOG.error("修改作品失败", e);
//            return new Result(false, "修改作品失败", e.getMessage());
//        }
//    }
//
//    @ApiOperation(value = "全部作品")
//    @RequestMapping(value = "all", method = RequestMethod.GET)
//    public Result<List<PaCreation>> all(PaCreation example) {
//        List<PaCreation> entitys = entityService.list(example);
//        if (null != entitys) {
//            return new Result<>(true, entitys);
//        }
//        return new Result<>(true, Collections.emptyList());
//    }

}
