package com.pixelai.api.picture.schedule;

import com.base.helper.StringUtils;
import com.pixelai.api.component.entity.CpPhoto;
import com.pixelai.api.component.service.CpPhotoService;
import com.pixelai.api.pa.entity.PaAiTryon;
import com.pixelai.api.pa.entity.PaCreation;
import com.pixelai.api.pa.entity.PaServices;
import com.pixelai.api.pa.entity.PaVipCurrency;
import com.pixelai.api.pa.service.PaAiTryonService;
import com.pixelai.api.pa.service.PaCreationService;
import com.pixelai.api.pa.service.PaServicesService;
import com.pixelai.api.pa.service.PaVipCurrencyService;
import com.pixelai.api.picture.util.AliyunOssUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Configuration;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.http.ResponseEntity;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.web.client.RestTemplate;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ExecutionException;

@Configuration
@EnableScheduling
public class ScheduledTasks {
    @Value("${aliyun.aliyunKey}")
    private String API_KEY;
    @Autowired
    RestTemplate restTemplate;
    @Autowired
    private PaCreationService paCreationService;
    @Autowired
    private CpPhotoService cpPhotoService;
    @Autowired
    private PaServicesService paServicesService;
    @Autowired
    private PaVipCurrencyService paVipCurrencyService;
    @Autowired
    private PaAiTryonService paAiTryonService;
    //每隔30秒拉取一次阿里云生成结果
//    @Scheduled(fixedRate = 30*1000)
    public void getCreationUrl() throws Exception {
        //4、5、6、7是阿里云服务
        List<PaCreation> paCreations= paCreationService.lambdaQuery()
                .eq(PaCreation::getState,"create")
                .in(PaCreation::getServiceId, Arrays.asList(4,5,6,7)).list();
        for (PaCreation paCreation : paCreations){
            //如果不存在任务id则生成失败，并返回钻石
            if(StringUtils.isEmpty(paCreation.getTaskId())){
                //更换失败图片
                updateUrlByPhotoId(paCreation,"https://guojunjie.oss-cn-hangzhou.aliyuncs.com/fail.jpg","fail");
                //退费
                PaServices paServices=paServicesService.lambdaQuery().eq(PaServices::getId,paCreation.getServiceId()).one();
                PaVipCurrency paVipCurrency=paVipCurrencyService.lambdaQuery().eq(PaVipCurrency::getUserid,paCreation.getUserId()).one();
                paVipCurrency.setNumerical(paVipCurrency.getNumerical()+paServices.getPrice());
                paVipCurrencyService.updateById(paVipCurrency);
                continue;
            }

            if(paCreation.getServiceId()==4){
                String url =getText2ImageByTaskId(paCreation.getTaskId());
                if (StringUtils.isNotEmpty(url)){
                    updateUrlByPhotoId(paCreation,url,"t");
                }
                continue;
            }
            //换衣
            if(paCreation.getServiceId()==5){
                String url =getTryonImageByTaskId(paCreation.getTaskId());
                if (StringUtils.isNotEmpty(url)){
                    //更新用户表
                    String ossUrl=updateUrlByPhotoId(paCreation,url,"t");
                    //根据作品id更新预览表
                    paAiTryonService.lambdaUpdate().set(PaAiTryon::getTargetUrl,ossUrl).eq(PaAiTryon::getCreationId,paCreation.getId()).update();
                }
                continue;
            }

            if(paCreation.getServiceId()==6){
                String url =getGeneration(paCreation.getTaskId());
                if (StringUtils.isNotEmpty(url)){
                    updateUrlByPhotoId(paCreation,url,"t");
                }
                continue;
            }

            if(paCreation.getServiceId()==7){
                String url =getText2ImageByTaskId(paCreation.getTaskId());
                if (StringUtils.isNotEmpty(url)){
                    updateUrlByPhotoId(paCreation,url,"t");
                }
                continue;
            }

        }
    }

    //AI换衣
    public String getTryonImageByTaskId(String taskId) throws Exception{
        String url="";
        HttpHeaders h = new HttpHeaders();
        h.set("Authorization", API_KEY);

        HttpEntity<Void> requestGetFromTask = new HttpEntity<>(h);
        ResponseEntity<Map> responseGetFromTask= restTemplate.exchange(
                "https://dashscope.aliyuncs.com/api/v1/tasks/{taskId}",
                HttpMethod.GET,
                requestGetFromTask,
                Map.class,
                taskId
        );
        Map NewResponse = responseGetFromTask.getBody();
        if ("SUCCEEDED".equals(((Map) NewResponse.get("output")).get("task_status"))) {//检查任务状态
            url=(String) ((Map) NewResponse.get("output")).get("image_url");//获取返回的阿里云云上的图片url
        }
        return url;
    }

    //人像风格重绘
    public String getGeneration(String taskId) throws Exception {
        String url="";
        HttpHeaders h = new HttpHeaders();
        h.set("Authorization", API_KEY);

        HttpEntity<Void> requestGetFromTask = new HttpEntity<>(h);
        ResponseEntity<Map> responseGetFromTask= restTemplate.exchange(
                "https://dashscope.aliyuncs.com/api/v1/tasks/{taskId}",
                HttpMethod.GET,
                requestGetFromTask,
                Map.class,
                taskId
        );
        Map NewResponse = responseGetFromTask.getBody();
        if ("SUCCEEDED".equals(((Map) NewResponse.get("output")).get("task_status"))) {//检查任务状态
            ArrayList<Map<String,String>> list=(ArrayList<Map<String,String>>) ((Map) NewResponse.get("output")).get("results");
            url=list.get(0).get("url");
        }
        return url;
    }


    public String updateUrlByPhotoId(PaCreation paCreation,String url,String state) throws IOException {
        MultipartFile file=AliyunOssUtil.fileUrlConvertToMultipartFile(url);
        String ossUrl=AliyunOssUtil.uploadFile(file);
        //更新图片url
        if (paCreation.getServiceId()==5){
            cpPhotoService.lambdaUpdate()
                    .set(CpPhoto::getCpLabelId,5)
                    .set(CpPhoto::getPath,ossUrl)
                    .set(CpPhoto::getState,state)
                    .eq(CpPhoto::getId,paCreation.getPhotoId()).update();
        }else{
            cpPhotoService.lambdaUpdate()
                    .set(CpPhoto::getPath,ossUrl)
                    .set(CpPhoto::getState,state)
                    .eq(CpPhoto::getId,paCreation.getPhotoId()).update();
        }
        //更新收藏品状态
        paCreationService.lambdaUpdate()
                .set(PaCreation::getState,state)
                .eq(PaCreation::getId,paCreation.getId()).update();
        return ossUrl;
    }

    //文字生成图片
    public String getText2ImageByTaskId(String taskId) throws Exception{
        String url="";
        HttpHeaders h = new HttpHeaders();
        h.set("Authorization", API_KEY);

        HttpEntity<Void> requestGetFromTask = new HttpEntity<>(h);
        ResponseEntity<Map> responseGetFromTask= restTemplate.exchange(
                "https://dashscope.aliyuncs.com/api/v1/tasks/{taskId}",
                HttpMethod.GET,
                requestGetFromTask,
                Map.class,
                taskId
        );
        Map NewResponse = responseGetFromTask.getBody();
        if ("SUCCEEDED".equals(((Map) NewResponse.get("output")).get("task_status"))) {//检查任务状态
            List<Map<String, String>> results=(List<Map<String, String>>) ((Map) NewResponse.get("output")).get("results");
            url=results.get(0).get("url");//获取返回的阿里云云上的图片url
        }
//        FAILED
        return url;
    }
}
