package com.pixelai.api.picture.service.impl;

import com.alibaba.dashscope.aigc.imagesynthesis.ImageSynthesis;
import com.alibaba.dashscope.aigc.imagesynthesis.ImageSynthesisParam;
import com.alibaba.dashscope.aigc.imagesynthesis.ImageSynthesisResult;
import com.alibaba.dashscope.exception.ApiException;
import com.alibaba.dashscope.exception.NoApiKeyException;
import com.alibaba.dashscope.utils.JsonUtils;
import com.alibaba.fastjson.JSON;
import com.pixelai.api.component.entity.CpPhoto;
import com.pixelai.api.component.service.CpPhotoService;
import com.pixelai.api.pa.entity.PaAiTryon;
import com.pixelai.api.pa.entity.PaCreation;
import com.pixelai.api.pa.service.PaAiTryonService;
import com.pixelai.api.pa.service.PaCreationService;
import com.pixelai.api.picture.config.VolcengineExConfig;
import com.pixelai.api.picture.service.AiPictureService;
import com.pixelai.api.picture.util.*;
import com.pixelai.api.picture.vo.AiPictureVo;
import com.pixelai.api.picture.vo.DirVo;
import com.pixelai.utils.StringUtil;
import okhttp3.*;
import okhttp3.MediaType;
import org.json.JSONArray;
import org.json.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.*;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;
import org.springframework.web.client.HttpClientErrorException;
import org.springframework.web.client.HttpServerErrorException;
import org.springframework.web.client.RestTemplate;
import org.springframework.web.multipart.MultipartFile;

import java.io.*;
import java.time.*;
import java.util.*;

import static com.pixelai.api.picture.util.AliyunOssUtil.fileUrlConvertToMultipartFile;

@Service
public class AiPictureServiceImpl implements AiPictureService {
    //指定生成图片路径
    @Value("${external.resource.dir}")
    private String externalResourceDir;
    @Value("${aliyun.aliyunKey}")
    private String API_KEY;
    @Value("${volcengine.AccessKeyID}")
    private String volcengineKEY;
    @Value("${volcengine.SecretAccessKey}")
    private String volcengineSecret;

    @Autowired
    private CpPhotoService cpPhotoService;
    @Autowired
    private PaCreationService paCreationService;
    @Autowired
    RestTemplate restTemplate;
    @Autowired
    private PaAiTryonService paAiTryonService;
    @Autowired
    private VolcengineExConfig volcengineExConfig;

    @Override
    public CpPhoto ImageColoring(AiPictureVo aiPictureVo, PaCreation paCreation) throws Exception {
        //请求百度URL
        String url="https://aip.baidubce.com/rest/2.0/image-process/v1/colourize";
        String base64=this.ActivateMagic(aiPictureVo.getImageUrl(),url,null);
        //保存base64为图片到oss
        MultipartFile img=AliyunOssUtil.base64ToMultipartFile(base64,System.currentTimeMillis()+"");
        String imgUrl=AliyunOssUtil.uploadFile(img);
        CpPhoto cpPhoto=cpPhotoService.getById(paCreation.getPhotoId());
        cpPhoto.setPath(imgUrl);
        cpPhotoService.updateById(cpPhoto);
        paCreation.setState("t");
        paCreationService.updateById(paCreation);
        return cpPhoto;
    }

    @Override
    public CpPhoto CharacterAnime(AiPictureVo aiPictureVo, PaCreation paCreation) throws Exception {
        //请求百度URL
        String url="https://aip.baidubce.com/rest/2.0/image-process/v1/selfie_anime";
        String base64=this.ActivateMagic(aiPictureVo.getImageUrl(),url,null);
        //保存base64为图片到oss
        MultipartFile img=AliyunOssUtil.base64ToMultipartFile(base64,System.currentTimeMillis()+"");
        String imgUrl=AliyunOssUtil.uploadFile(img);
        CpPhoto cpPhoto=cpPhotoService.getById(paCreation.getPhotoId());
        cpPhoto.setPath(imgUrl);
        cpPhotoService.updateById(cpPhoto);
        paCreation.setState("t");
        paCreationService.updateById(paCreation);
        return cpPhoto;
    }

    @Override
    public CpPhoto styleTrans(AiPictureVo aiPictureVo, PaCreation paCreation) throws Exception {
        //请求百度URL
        String url="https://aip.baidubce.com/rest/2.0/image-process/v1/style_trans";
        String base64=this.ActivateMagic(aiPictureVo.getImageUrl(),url,aiPictureVo.getOption());
        //保存base64为图片到oss
        MultipartFile img=AliyunOssUtil.base64ToMultipartFile(base64,System.currentTimeMillis()+"");
        String imgUrl=AliyunOssUtil.uploadFile(img);
        CpPhoto cpPhoto=cpPhotoService.getById(paCreation.getPhotoId());
        cpPhoto.setPath(imgUrl);
        cpPhotoService.updateById(cpPhoto);
        paCreation.setState("t");
        paCreationService.updateById(paCreation);
        return cpPhoto;
    }

    @Async
    @Override
    public void textToImage(AiPictureVo aiPictureVo,PaCreation paCreation) throws Exception {
        //异步请求阿里云URL，获取任务id
        String taskId=((Map)this.text2Images(aiPictureVo.getText(),aiPictureVo.getOption())).get("task_id").toString();
        paCreation.setTaskId(taskId);
        paCreationService.updateById(paCreation);
    }

    @Override
    @Async
    public void textToImageByVolcengine(AiPictureVo aiPictureVo, PaCreation paCreation) throws Exception {
        Map response =TextToImage(aiPictureVo);
        JSONObject jsonObject = new JSONObject(response.get("body").toString());
        JSONObject data=jsonObject.getJSONObject("data");
        System.out.printf(data.getString("llm_result"));
        JSONArray imageUrls=data.getJSONArray("image_urls");
        String imageUrl=imageUrls.get(0).toString();
        //业务代码
        MultipartFile image=AliyunOssUtil.fileUrlConvertToMultipartFile(imageUrl);
        imageUrl=AliyunOssUtil.uploadFile(image);
        CpPhoto cpPhoto=cpPhotoService.getById(paCreation.getPhotoId());
        cpPhoto.setPath(imageUrl);
        cpPhoto.setName(aiPictureVo.getText());
        cpPhotoService.updateById(cpPhoto);
        paCreation.setState("t");
        paCreationService.updateById(paCreation);
    }

    public Map<String, Object> TextToImage(AiPictureVo aiPictureVo) throws Exception {
        String endpoint = "visual.volcengineapi.com";//访问地址
        String path = "/"; //访问路径url
        String service = "cv";//服务，默认cv
        String region = "cn-north-1";//服务地址
        String schema = "https";//默认https
        String action = "CVProcess";
        String version = "2022-08-31";
        // 假设你已经有了一个 LocalTime 对象
        LocalTime localTime = LocalTime.now();

        // 使用当前日期和给定的时间创建 LocalDateTime
        LocalDateTime localDateTime = LocalDate.now().atTime(localTime);

        // 将 LocalDateTime 转换为 Instant
        Instant instant = localDateTime.atZone(ZoneId.systemDefault()).toInstant();

        // 将 Instant 转换为 java.util.Date
        Date date = Date.from(instant);

        HashMap<String, String> queryMap = new HashMap<>();
        com.alibaba.fastjson.JSONObject req=new com.alibaba.fastjson.JSONObject();
        req.put("req_key", "high_aes_general_v21_L");
        req.put("prompt", aiPictureVo.getText());
        req.put("req_schedule_conf", aiPictureVo.getOption());//标准版：general_v20_9B_rephraser 美感版：general_v20_9B_pe
        req.put ("model_version", "general_v2.0_L");
        req.put ("return_url", true);
        req.put ("use_pre_llm", false);

        VolcengineUtil volcengineUtil=new VolcengineUtil(region, service, schema, endpoint, path, volcengineKEY, volcengineSecret);
        return  volcengineUtil.doRequest("POST", new HashMap(), req.toJSONString().getBytes(), date, action, version);
    }

    @Override
    @Async
    public void aiTryonPreview(AiPictureVo aiPictureVo, PaCreation paCreation) throws Exception {
        PaAiTryon paAiTryon=paAiTryonService.lambdaQuery()
                .eq(PaAiTryon::getImageUrl,aiPictureVo.getImageUrl())
                .eq(PaAiTryon::getTopImageUrl,aiPictureVo.getTopImageUrl())
                .eq(StringUtil.isNotBlank(aiPictureVo.getBottomImageUrl()),PaAiTryon::getBottomImageUrl,aiPictureVo.getBottomImageUrl()).one();
        //如果不存在换衣记录，则请求阿里云获取并记录
        if (paAiTryon==null){
            //异步请求阿里云URL，获取任务id，tryon传参，上衣、下衣、人物
            String taskId=((Map)this.tryon(aiPictureVo.getTopImageUrl(),aiPictureVo.getBottomImageUrl(),aiPictureVo.getImageUrl())).get("task_id").toString();
            paCreation.setTaskId(taskId);
            paCreationService.updateById(paCreation);
            //保存换衣记录
            paAiTryonService.save(new PaAiTryon(aiPictureVo.getImageUrl(),aiPictureVo.getTopImageUrl(),aiPictureVo.getBottomImageUrl(),null,paCreation.getId()));
            return;
        }
        //如果存在换衣记录，则置换path，并标记生成成功
        cpPhotoService.lambdaUpdate()
                .set(CpPhoto::getPath,paAiTryon.getTargetUrl())
                .eq(CpPhoto::getId,paCreation.getPhotoId())
                .update();
        paCreation.setState("t");
        paCreationService.updateById(paCreation);
    }

    @Override
    @Async
    public void aiTryon(AiPictureVo aiPictureVo, PaCreation paCreation) throws Exception {
        //异步请求阿里云URL，获取任务id，tryon传参，上衣、下衣、人物
        String taskId=((Map)this.tryon(aiPictureVo.getTopImageUrl(),aiPictureVo.getBottomImageUrl(),aiPictureVo.getImageUrl())).get("task_id").toString();
        paCreation.setTaskId(taskId);
        paCreationService.updateById(paCreation);
    }

    @Override
    @Async
    public void portraitStyleRepainting(AiPictureVo aiPictureVo, PaCreation paCreation) throws Exception {
        //异步请求阿里云URL，获取任务id
        String taskId=((Map)this.generation(aiPictureVo.getImageUrl(),aiPictureVo.getOption())).get("task_id").toString();
        paCreation.setTaskId(taskId);
        paCreationService.updateById(paCreation);
    }

    @Override
    @Async
    public void texture(AiPictureVo aiPictureVo, PaCreation paCreation) throws Exception {
        //异步请求阿里云URL，获取任务id
        String taskId=((Map)this.textureImage(aiPictureVo)).get("task_id").toString();
        paCreation.setTaskId(taskId);
        paCreationService.updateById(paCreation);
    }

    @Override
    @Async
    public void ageChange(AiPictureVo aiPictureVo, PaCreation paCreation) throws Exception {
        Map response = ageChange(aiPictureVo);
        JSONObject jsonObject = new JSONObject(response.get("body").toString());
        JSONObject data=jsonObject.getJSONObject("data");
        JSONArray imageUrls=data.getJSONArray("image_urls");
        String imageUrl=imageUrls.get(0).toString();
        //业务代码
        MultipartFile image=AliyunOssUtil.fileUrlConvertToMultipartFile(imageUrl);
        imageUrl=AliyunOssUtil.uploadFile(image);
        CpPhoto cpPhoto=cpPhotoService.getById(paCreation.getPhotoId());
        cpPhoto.setPath(imageUrl);
        cpPhotoService.updateById(cpPhoto);
        paCreation.setState("t");
        paCreationService.updateById(paCreation);
    }

    @Override
    @Async
    public void threeDGameCartoon(AiPictureVo aiPictureVo, PaCreation paCreation) throws Exception {
        Map response = threeDGameCartoon(aiPictureVo);
        JSONObject jsonObject = new JSONObject(response.get("body").toString());
        JSONObject data=jsonObject.getJSONObject("data");
        JSONArray imageUrls=data.getJSONArray("image_urls");
        String imageUrl=imageUrls.get(0).toString();
        //业务代码
        MultipartFile image=AliyunOssUtil.fileUrlConvertToMultipartFile(imageUrl);
        imageUrl=AliyunOssUtil.uploadFile(image);
        CpPhoto cpPhoto=cpPhotoService.getById(paCreation.getPhotoId());
        cpPhoto.setPath(imageUrl);
        cpPhotoService.updateById(cpPhoto);
        paCreation.setState("t");
        paCreationService.updateById(paCreation);
    }

    private Map threeDGameCartoon(AiPictureVo aiPictureVo) throws Exception {
        String AccessKeyID = volcengineExConfig.getAccessKeyID();// apikey
        String SecretAccessKey = volcengineExConfig.getSecretAccessKey(); //密钥
        String endpoint = volcengineExConfig.getEndpoint();//访问地址
        String path = volcengineExConfig.getPath(); //访问路径url
        String service = volcengineExConfig.getService();//服务，默认cv
        String region = volcengineExConfig.getRegion();//服务地址
        String schema = volcengineExConfig.getSchema();//默认https
        String action = "3DGameCartoon";
        String version = "2020-08-26";

        // 假设你已经有了一个 LocalTime 对象
        LocalTime localTime = LocalTime.now();

        // 使用当前日期和给定的时间创建 LocalDateTime
        LocalDateTime localDateTime = LocalDate.now().atTime(localTime);

        // 将 LocalDateTime 转换为 Instant
        Instant instant = localDateTime.atZone(ZoneId.systemDefault()).toInstant();

        // 将 Instant 转换为 java.util.Date
        Date date = Date.from(instant);

        com.alibaba.fastjson.JSONObject req=new com.alibaba.fastjson.JSONObject();
//        req.put("image_base64",volThreeDGameCartoonVo.getImage_base64());
        req.put("image_url",aiPictureVo.getImageUrl());
//        req.put("return_url", true);

        VolcengineUtil volcengineUtil=new VolcengineUtil(region, service, schema, endpoint, path, AccessKeyID, SecretAccessKey);
        return  volcengineUtil.doRequest("POST", new HashMap(), req.toJSONString().getBytes(), date, action, version);
    }

    private Map ageChange(AiPictureVo aiPictureVo) throws Exception {
        String AccessKeyID = volcengineExConfig.getAccessKeyID();// apikey
        String SecretAccessKey = volcengineExConfig.getSecretAccessKey(); //密钥
        String endpoint = volcengineExConfig.getEndpoint();//访问地址
        String path = volcengineExConfig.getPath(); //访问路径url
        String service = volcengineExConfig.getService();//服务，默认cv
        String region = volcengineExConfig.getRegion();//服务地址
        String schema = volcengineExConfig.getSchema();//默认https

        String action = "AllAgeGeneration";
        String version = "2022-08-31";

        // 假设你已经有了一个 LocalTime 对象
        LocalTime localTime = LocalTime.now();

        // 使用当前日期和给定的时间创建 LocalDateTime
        LocalDateTime localDateTime = LocalDate.now().atTime(localTime);

        // 将 LocalDateTime 转换为 Instant
        Instant instant = localDateTime.atZone(ZoneId.systemDefault()).toInstant();

        // 将 Instant 转换为 java.util.Date
        Date date = Date.from(instant);

        com.alibaba.fastjson.JSONObject req=new com.alibaba.fastjson.JSONObject();
        req.put("req_key", "all_age_generation");
        req.put("image_urls",Arrays.asList(aiPictureVo.getImageUrl()));
        req.put("target_age", Integer.valueOf(aiPictureVo.getOption()));
        req.put("do_risk", false);
        req.put("return_url", true);
        VolcengineUtil volcengineUtil=new VolcengineUtil(region, service, schema, endpoint, path, AccessKeyID, SecretAccessKey);
        return  volcengineUtil.doRequest("POST", new HashMap(), req.toJSONString().getBytes(), date, action, version);
    }
    @Override
    @Async
    public void emotionPortrait(AiPictureVo aiPictureVo, PaCreation paCreation) throws Exception {
        Map response = emotionPortrait(aiPictureVo);
        JSONObject jsonObject = new JSONObject(response.get("body").toString());
        JSONObject data=jsonObject.getJSONObject("data");
        JSONArray imageUrls=data.getJSONArray("image_urls");
        String imageUrl=imageUrls.get(0).toString();
        //业务代码
        MultipartFile image=AliyunOssUtil.fileUrlConvertToMultipartFile(imageUrl);
        imageUrl=AliyunOssUtil.uploadFile(image);
        CpPhoto cpPhoto=cpPhotoService.getById(paCreation.getPhotoId());
        cpPhoto.setPath(imageUrl);
        cpPhotoService.updateById(cpPhoto);
        paCreation.setState("t");
        paCreationService.updateById(paCreation);
    }

    @Override
    @Async
    public void hairStyle(AiPictureVo aiPictureVo, PaCreation paCreation) throws Exception {
        Map response = hairStyle(aiPictureVo);
        JSONObject jsonObject = new JSONObject(response.get("body").toString());
        JSONObject data=jsonObject.getJSONObject("data");
        JSONArray imageUrls=data.getJSONArray("image_urls");
        String imageUrl=imageUrls.get(0).toString();
        //业务代码
        MultipartFile image=AliyunOssUtil.fileUrlConvertToMultipartFile(imageUrl);
        imageUrl=AliyunOssUtil.uploadFile(image);
        CpPhoto cpPhoto=cpPhotoService.getById(paCreation.getPhotoId());
        cpPhoto.setPath(imageUrl);
        cpPhotoService.updateById(cpPhoto);
        paCreation.setState("t");
        paCreationService.updateById(paCreation);
    }

    public Map<String, Object> hairStyle(AiPictureVo aiPictureVo) throws Exception{
        String endpoint = "visual.volcengineapi.com";//访问地址
        String path = "/"; //访问路径url
        String service = "cv";//服务，默认cv
        String region = "cn-north-1";//服务地址
        String schema = "https";//默认https
        String action = "CVProcess";
        String version = "2022-08-31";
        // 假设你已经有了一个 LocalTime 对象
        LocalTime localTime = LocalTime.now();

        // 使用当前日期和给定的时间创建 LocalDateTime
        LocalDateTime localDateTime = LocalDate.now().atTime(localTime);

        // 将 LocalDateTime 转换为 Instant
        Instant instant = localDateTime.atZone(ZoneId.systemDefault()).toInstant();

        // 将 Instant 转换为 java.util.Date
        Date date = Date.from(instant);

        com.alibaba.fastjson.JSONObject req=new com.alibaba.fastjson.JSONObject();
        req.put("req_key", "hair_style");
        req.put("image_urls",Arrays.asList(aiPictureVo.getImageUrl()));
        req.put("hair_type", Integer.valueOf(aiPictureVo.getOption()));
        req.put("return_url",true);

        VolcengineUtil volcengineUtil=new VolcengineUtil(region, service, schema, endpoint, path, volcengineKEY, volcengineSecret);
        return  volcengineUtil.doRequest("POST", new HashMap(), req.toJSONString().getBytes(), date, action, version);
    }

    private Map<String, Object> emotionPortrait(AiPictureVo aiPictureVo) throws Exception {
        String endpoint = "visual.volcengineapi.com";//访问地址
        String path = "/"; //访问路径url
        String service = "cv";//服务，默认cv
        String region = "cn-north-1";//服务地址
        String schema = "https";//默认https
        String action = "CVProcess";
        String version = "2022-08-31";
        // 假设你已经有了一个 LocalTime 对象
        LocalTime localTime = LocalTime.now();

        // 使用当前日期和给定的时间创建 LocalDateTime
        LocalDateTime localDateTime = LocalDate.now().atTime(localTime);

        // 将 LocalDateTime 转换为 Instant
        Instant instant = localDateTime.atZone(ZoneId.systemDefault()).toInstant();

        // 将 Instant 转换为 java.util.Date
        Date date = Date.from(instant);

        HashMap<String, String> queryMap = new HashMap<>();
        com.alibaba.fastjson.JSONObject req=new com.alibaba.fastjson.JSONObject();
        req.put("req_key", "emotion_portrait");
        req.put("image_urls",Arrays.asList(aiPictureVo.getImageUrl()));
        req.put("target_emotion",aiPictureVo.getOption());
        req.put("return_url",true);
        VolcengineUtil volcengineUtil=new VolcengineUtil(region, service, schema, endpoint, path, volcengineKEY, volcengineSecret);
        return  volcengineUtil.doRequest("POST", new HashMap(), req.toJSONString().getBytes(), date, action, version);
    }

    private Map textureImage(AiPictureVo aiPictureVo){
        // 设置HTTP头
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(org.springframework.http.MediaType.APPLICATION_JSON);//固定，返回类型，只有json
        headers.set("Authorization", API_KEY);//固定，api密钥
        headers.set("X-DashScope-Async", "enable");//只有默认，异步提交任务
        String url="https://dashscope.aliyuncs.com/api/v1/services/aigc/wordart/texture";//风格转换url
        Map<String, Object> text=new HashMap<>();/////////////////input中的text对象
        text.put("text_content",aiPictureVo.getText());//设置文字内容
//        text.put("font_name",aiPictureVo.getOption() );//设置文字样式
//        text.put("output_image_ratio",output_image_ratio);//设置图片中文字比例
        Map<String, Object> input = new HashMap<>();//放到request中
        input.put("text", text);//////////将text对象放进来
        input.put("prompt", aiPictureVo.getPromt());//////////提示词
        input.put("texture_style", aiPictureVo.getOption());//////////样式
        Map<String,Object> parameters=new HashMap<>();
        parameters.put("image_short_size",704);//默认尺寸
        parameters.put("n",1);//默认一张
        parameters.put("alpha_channel",false);//默认不带alpha通道
        Map<String, Object> request = new HashMap<>();//请求体
        request.put("model", "wordart-texture");/////////选择模型wordart-texture，一个apikey可以冲所有模型，详情看文档介绍
        request.put("input", input);/////////input放进来
        request.put("parameters", parameters);/////////parameters对象
        HttpEntity<Map<String, Object>> entity = new HttpEntity<>(request, headers);//生成一个请求内容
        ResponseEntity<Map> response = restTemplate.postForEntity(url, entity, Map.class);//发送
        return (Map)response.getBody().get("output");
    }
    public Map generation(String image_Url,String option) throws Exception {
        // 设置HTTP头
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(org.springframework.http.MediaType.APPLICATION_JSON);//固定，返回类型，只有json
        headers.set("Authorization", API_KEY);//固定，api密钥
        headers.set("X-DashScope-Async", "enable");//只有默认，异步提交任务
        String url="https://dashscope.aliyuncs.com/api/v1/services/aigc/image-generation/generation";//风格转换url
        Map<String, Object> input = new HashMap<>();//放到request中
        input.put("image_url", image_Url);//////////图片url
        // "https://c-ssl.duitang.com/uploads/item/201410/23/20141023222303_4UYNu.jpeg"测试用图
        input.put("style_index", Integer.valueOf(option));//////////样式
        Map<String, Object> request = new HashMap<>();//请求体
        request.put("model", "wanx-style-repaint-v1");/////////选择模型wanx-style-repaint-v1，一个apikey可以冲所有模型，详情看文档介绍
        request.put("input", input);/////////文本描述和尺寸
        HttpEntity<Map<String, Object>> entity = new HttpEntity<>(request, headers);//生成一个请求内容
        ResponseEntity<Map> response = restTemplate.postForEntity(url, entity, Map.class);//发送
        return (Map)response.getBody().get("output");
    }

    public Map tryon(String top_garment_url, String bottom_garment_url, String person_image_url) throws Exception {
        // 设置HTTP头
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(org.springframework.http.MediaType.APPLICATION_JSON);
        headers.set("Authorization", "Bearer " + API_KEY); // 注意这里应该是Bearer token格式
        headers.set("X-DashScope-Async", "enable");

        // 构建请求体
        Map<String, Object> input = new HashMap<>();
        input.put("top_garment_url", top_garment_url);
        if (bottom_garment_url!=null)
            input.put("bottom_garment_url",bottom_garment_url);
        input.put("person_image_url", person_image_url);


        Map<String, Object> parameters = new HashMap<>();
        parameters.put("resolution", -1); // 确认这是API接受的有效值
        parameters.put("restore_face", true);

        Map<String, Object> request = new HashMap<>();
        request.put("model", "aitryon");
        request.put("input", input);
        request.put("parameters", parameters);
        String url = "https://dashscope.aliyuncs.com/api/v1/services/aigc/image2image/image-synthesis/";

        HttpEntity<Map<String, Object>> entity = new HttpEntity<>(request, headers);

        try {
            ResponseEntity<Map> response = restTemplate.postForEntity(url, entity, Map.class);

            // 检查响应状态码是否为200 OK
            if (response.getStatusCode() == HttpStatus.OK) {
                return (Map)response.getBody().get("output");
            } else {
                throw new RuntimeException("无法获取返回: " + response.getStatusCode());
            }
        } catch (HttpClientErrorException e) {
            System.err.println(e.getResponseBodyAsString());
            throw e;
        } catch (HttpServerErrorException e) {
            System.err.println(e.getResponseBodyAsString());
            throw e;
        }
    }



    public Map text2Images(String prompt,String option) throws Exception {
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(org.springframework.http.MediaType.APPLICATION_JSON);//固定，返回类型，只有json
        headers.set("Authorization", API_KEY);//固定，api密钥
        headers.set("X-DashScope-Async", "enable");//只有默认，异步提交任务
        String url="https://dashscope.aliyuncs.com/api/v1/services/aigc/text2image/image-synthesis";//文生图url
        Map<String, Object> input = new HashMap<>();//放到request中
        input.put("prompt", prompt);//////////文本描述
        input.put("size", 1024*1024);//////////尺寸
        Map<String, Object> parameters = new HashMap<>();//放到request中
        parameters.put("style", option);//////////类型
        parameters.put("n", 1);//////////数量
        Map<String, Object> request = new HashMap<>();//请求体
        request.put("model", "wanx-v1");/////////选择模型wanx-v1，一个apikey可以冲所有模型，详情看文档介绍
        request.put("input", input);/////////文本描述和尺寸
        request.put("parameters", parameters);//////////类型和数量
        HttpEntity<Map<String, Object>> entity = new HttpEntity<>(request, headers);//生成一个请求内容
        ResponseEntity<Map> response = restTemplate.postForEntity(url, entity, Map.class);//发送
        return (Map)response.getBody().get("output");
    }

    public String getGeneration(String taskId) throws Exception {
        String url="";
        HttpHeaders h = new HttpHeaders();
        h.set("Authorization", API_KEY);

        HttpEntity<Void> requestGetFromTask = new HttpEntity<>(h);
        ResponseEntity<Map> responseGetFromTask= restTemplate.exchange(
                "https://dashscope.aliyuncs.com/api/v1/tasks/{taskId}",
                HttpMethod.GET,
                requestGetFromTask,
                Map.class,
                taskId
        );
        Map NewResponse = responseGetFromTask.getBody();
        if ("SUCCEEDED".equals(((Map) NewResponse.get("output")).get("task_status"))) {//检查任务状态
            ArrayList<Map<String,String>> list=(ArrayList<Map<String,String>>) ((Map) NewResponse.get("output")).get("results");
            url=list.get(0).get("url");
        }
        return url;
    }

    public String getTryonImageByTaskId(String taskId) throws Exception{
        String url="";
        HttpHeaders h = new HttpHeaders();
        h.set("Authorization", API_KEY);

        HttpEntity<Void> requestGetFromTask = new HttpEntity<>(h);
        ResponseEntity<Map> responseGetFromTask= restTemplate.exchange(
                "https://dashscope.aliyuncs.com/api/v1/tasks/{taskId}",
                HttpMethod.GET,
                requestGetFromTask,
                Map.class,
                taskId
        );
        Map NewResponse = responseGetFromTask.getBody();
        if ("SUCCEEDED".equals(((Map) NewResponse.get("output")).get("task_status"))) {//检查任务状态
            url=(String) ((Map) NewResponse.get("output")).get("image_url");//获取返回的阿里云云上的图片url
        }
        return url;
    }

    public String getText2ImageByTaskId(String taskId) throws Exception{
        String url="";
        HttpHeaders h = new HttpHeaders();
        h.set("Authorization", API_KEY);

        HttpEntity<Void> requestGetFromTask = new HttpEntity<>(h);
        ResponseEntity<Map> responseGetFromTask= restTemplate.exchange(
                "https://dashscope.aliyuncs.com/api/v1/tasks/{taskId}",
                HttpMethod.GET,
                requestGetFromTask,
                Map.class,
                taskId
        );
        Map NewResponse = responseGetFromTask.getBody();
        if ("SUCCEEDED".equals(((Map) NewResponse.get("output")).get("task_status"))) {//检查任务状态
            List<Map<String, String>> results=(List<Map<String, String>>) ((Map) NewResponse.get("output")).get("results");
            url=results.get(0).get("url");//获取返回的阿里云云上的图片url
        }
        return url;
    }



    public String basicCall(String prompt,String option) throws ApiException, NoApiKeyException {
        ImageSynthesisParam param =
                ImageSynthesisParam.builder()
                        .apiKey(API_KEY)
                        .model(ImageSynthesis.Models.WANX_V1)
                        .prompt(prompt)
                        .style(option)
                        .n(1)
                        .size("1024*1024")
                        .build();

        ImageSynthesis imageSynthesis = new ImageSynthesis();
        ImageSynthesisResult result = null;
        try {
            result = imageSynthesis.call(param);
        } catch (ApiException | NoApiKeyException e){
            throw new RuntimeException(e.getMessage());
        }
        return JsonUtils.toJson(result);
    }


    public DirVo getDir(Integer userId){
        // 生成新的文件名
        String timestamp = System.currentTimeMillis()+"";
        String fileNameSource=timestamp+"Source.png";
        String fileNameSource2=timestamp+"Source2.png";
        String fileNameSource3=timestamp+"Source3.png";
        String fileNameTarget=timestamp+"Target.png";

        //创建文件夹
        File fileDir =new File(externalResourceDir+"\\"+userId);
        if (!fileDir.exists() && !fileDir.isDirectory()) {
            fileDir.mkdirs();
        }

        //文件存储路径
        String pathSouce=externalResourceDir+"\\"+userId+"\\"+fileNameSource;
        String pathSouce2=externalResourceDir+"\\"+userId+"\\"+fileNameSource2;
        String pathSouce3=externalResourceDir+"\\"+userId+"\\"+fileNameSource3;
        String pathTarget=externalResourceDir+"\\"+userId+"\\"+fileNameTarget;
        DirVo dirVo=new DirVo(timestamp,fileNameSource,fileNameSource2,fileNameSource3,fileNameTarget,pathSouce,pathSouce2,pathSouce3,pathTarget);
        return dirVo;
    }

    public String getBaiduAiToken() throws IOException {
        OkHttpClient HTTP_CLIENT = new OkHttpClient().newBuilder().build();
        MediaType mediaType = MediaType.parse("application/json");
        RequestBody body = RequestBody.create(mediaType, "");
        Request request = new Request.Builder()
                .url("https://aip.baidubce.com/oauth/2.0/token?client_id=VLIGkTb90ycaA0RxgOwzAlFm&client_secret=zVbIP2i70weJ3vQ9NUqeindPMDW28KDl&grant_type=client_credentials")
                .method("POST", body)
                .addHeader("Content-Type", "application/json")
                .addHeader("Accept", "application/json")
                .build();
        Response response = HTTP_CLIENT.newCall(request).execute();
//        System.out.println(response.body().string());
        HashMap hashMap = JSON.parseObject(response.body().string(), HashMap.class);
        String token = hashMap.get("access_token").toString();
        return token;
    }



    @Override//图片转base64传给baiduapi解析后获取传回来的base64码,黑白图片上色
    public String ActivateMagic(String imgUrl,String url,String option) throws Exception {//转码传输到baiduapi
            String param;
            if (option!="")
                param =  "option=" + option +"&url=" + imgUrl;
            else
                param = "url=" + imgUrl;
            String accessToken =getBaiduAiToken();
            String result = HttpUtil.post(url, accessToken, param);

            // 解析JSON并提取image属性
            JSONObject jsonObject = new JSONObject(result);
            String base64 = jsonObject.getString("image");
            return base64;
    }

    @Override//将转存好的base64码转为图片
     public String Base64ToImage(String outputImagePath,String base64String){
        // 输入 Base64 编码文件的路径
//        String outputImagePath =path+".png";
        try {
            byte[] imageBytes = Base64.getDecoder().decode(base64String);
            try (FileOutputStream fos = new FileOutputStream(outputImagePath)) {
                fos.write(imageBytes);
                System.out.println("图像已成功保存到 " + outputImagePath);
                return outputImagePath;
            }
        } catch (Exception e) {
            System.err.println("解码失败，可能是因为数据不完整或格式错误: " + e.getMessage());
        }
        return null;
    }


}
