package com.pixelai.api.picture.util;

import com.aliyun.oss.OSSClient;
import com.aliyun.oss.model.ObjectMetadata;
import io.undertow.server.handlers.form.FormData;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.mock.web.MockMultipartFile;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.commons.CommonsMultipartFile;

import java.io.*;
import java.net.HttpURLConnection;
import java.net.URL;
import java.net.URLConnection;
import java.security.SecureRandom;
import java.text.SimpleDateFormat;
import java.util.Base64;
import java.util.Date;

public class AliyunOssUtil {
    private static String ALIYUN_OSS_ENDPOINT="oss-cn-hangzhou.aliyuncs.com";
    private static String ALIYUN_OSS_ACCESSKEYID="LTAI5tFAiAq9SjM9vaucMuYD";
    private static String ALIYUN_OSS_ACCESSKEYSECRET="RHDmRIOaYlEXe9t1ro7ls8Ejcnhigv";
    private static String ALIYUN_OSS_BUCKETNAME="guojunjie";

    public static  String  uploadFile(MultipartFile multipartFile) throws IOException {
//        try {
            OSSClient ossClient = new OSSClient(ALIYUN_OSS_ENDPOINT, ALIYUN_OSS_ACCESSKEYID, ALIYUN_OSS_ACCESSKEYSECRET);
            SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd");
            // 获取文件名
            String fileName = multipartFile.getOriginalFilename();
            // 获取文件后缀名
            String suffixName = fileName.substring(fileName.lastIndexOf("."));
            // 最后上传生成的文件名
            String finalFileName = System.currentTimeMillis() + "" + new SecureRandom().nextInt(0x0400) + suffixName;
            // oss中的文件夹名
            String objectName = sdf.format(new Date()) + "/" + finalFileName;
            // 创建上传文件的元信息，可以通过文件元信息设置HTTP header(设置了才能通过返回的链接直接访问)。
            ObjectMetadata objectMetadata = new ObjectMetadata();
            objectMetadata.setContentType("image/jpg");
            // 文件上传
            ossClient.putObject(ALIYUN_OSS_BUCKETNAME, objectName, new ByteArrayInputStream(multipartFile.getBytes()),objectMetadata);
            // 设置URL过期时间为10年。
            Date expiration = new Date(System.currentTimeMillis() + 3600 * 1000*24*365*10);
            String url = ossClient.generatePresignedUrl(ALIYUN_OSS_BUCKETNAME, objectName, expiration).toString();
            ossClient.shutdown();
//        } catch (Exception e) {
//            System.out.println(e.toString());
//        }finally {
//            ossClient.shutdown();
//        }

//        log.info("阿里云OSS的文件url:{}", url);
        return url;
    }


    /**
     * base64转MultipartFile
     * @param base64String base64字符串
     * @param fileName 文件名称
     * @return
     * @throws IOException
     */
    public static MultipartFile base64ToMultipartFile(String base64String,String fileName ) {
        // 解码base64字符串 （如果base64有前缀如：data:image/jpg;base64,则需要去掉前缀，我这里有前缀，所以需要去掉）
        //有前缀
        //byte[] decode = Base64.getDecoder().decode(base64String.split(",")[1]);
        //无前缀
        byte[] decode = Base64.getDecoder().decode(base64String);
        // 创建字符串
        String name = fileName +  ".png"; // 文件名
        String contentType = "image/png"; // 内容类型
        return new MockMultipartFile(fileName, name, contentType, decode);
    }

    /**
     * 将网络文件转换为文件流
     * @param imageUrl
     * @return
     */
    public static MultipartFile fileUrlConvertToMultipartFile(String imageUrl) {
        try {
            // 将在线图片地址转换为URL对象
            URL url = new URL(imageUrl);
            // 打开URL连接
            URLConnection connection = url.openConnection();
            // 转换为HttpURLConnection对象
            HttpURLConnection httpURLConnection = (HttpURLConnection) connection;
            // 获取输入流
            InputStream inputStream = httpURLConnection.getInputStream();
            // 读取输入流中的数据，并保存到字节数组中
            ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
            byte[] buffer = new byte[1024];
            int bytesRead;
            while ((bytesRead = inputStream.read(buffer)) != -1) {
                byteArrayOutputStream.write(buffer, 0, bytesRead);
            }
            // 将字节数组转换为字节数组
            byte[] bytes = byteArrayOutputStream.toByteArray();
            // 创建ByteArrayInputStream对象，将字节数组传递给它
            ByteArrayInputStream byteArrayInputStream = new ByteArrayInputStream(bytes);
            // 创建MultipartFile对象，将ByteArrayInputStream对象作为构造函数的参数
            MultipartFile multipartFile = new MockMultipartFile("file", "filename.jpg", "image/jpg", byteArrayInputStream);
            return multipartFile;
        }catch (IOException ex){
            ex.printStackTrace();
            throw new RuntimeException("上传异常");
        }
    }



}
