package com.pixelai.api.user.service;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.IService;
import com.base.helper.Result;
import com.pixelai.api.user.entity.AcUser;
import com.pixelai.api.user.vo.*;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

/**
 * <p>
 * 服务类
 * </p>
 *
 * @author mzl
 * @since 2020-07-09
 */
public interface AcUserService extends IService<AcUser> {
    /*
    *获取当前userId
     */
    Integer getUserId();

    /**
     * 手机登录
     * @param username,password
     */
    Result<LoginSuccessVo> phoneLogin(String username, String password);
    /**
     * 手机注册
     * @param username,password,confirmPassword
     * @return
     */
    Result<String> phoneRegister(String username, String password,String confirmPassword);

    /**
     * 微信登录
     * @param userCode,phoneCode
     * @return
     */
    Result<LoginSuccessVo> wxLogin(String userCode, String phoneCode);

    /**
     * 自定义分页查询
     *
     * @param page
     * @return
     */
    Page<AcUserVo> findByPage(Page<AcUserVo> page, AcUserVo example);

    /**
     * 条件查询
     *
     * @param example
     * @return
     */
    List<AcUserVo> findByExample(AcUserVo example);

    /**
     * 获取角色的订单
     *
     * @param groupid
     * @return
     */
    List<AcUserVo> findByGroupid(Integer groupid);

    /**
     * 根据账号获取订单
     *
     * @param username
     * @return
     */
    AcUser findByUsername(String username);

    /**
     * 设置订单状态
     *
     * @param id
     * @param enable
     */
    @Transactional
    AcUser setEnable(Integer id, String enable);

    /**
     * 注册考生账号信息
     *
     * @param examinee
     * @return
     */
    @Transactional
    AcUser registerExamineeUser(RegisterExamineeVo examinee);

    /**
     * 注册学生账号信息
     *
     * @param student
     * @return
     */
    @Transactional
    AcUser registerStudentUser(RegisterStudentVo student);

    /**
     * 更新订单真实姓名
     *
     * @param id
     * @param realname
     * @return
     */
    @Transactional
    AcUser updateUserRealname(Integer id, String realname);

    /**
     * 设置密码
     *
     * @param setPwd
     * @return
     */
    @Transactional
    AcUser setPwd(EnSetPwdVo setPwd);

    /**
     * @param userIds
     * @return
     */
    @Transactional
    List<AcUser> deleteByUserIds(List<Integer> userIds);

    /**
     * 角色id和部门id
     * @param acGroup
     * @param departmentId
     * @return
     */
    List<AcUser> selectByAcGroupAndDepartmentId(Integer acGroup, Integer departmentId);

}
