package com.pixelai.utils;

import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONUtil;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Component;

import java.util.concurrent.TimeUnit;
import java.util.function.Function;

import static com.pixelai.utils.RedisConstants.CACHE_NULL_TTL;

@Component
public class CacheClient {

    private final StringRedisTemplate stringRedisTemplate;

    public CacheClient(StringRedisTemplate stringRedisTemplate){
        this.stringRedisTemplate = stringRedisTemplate;
    }

    public void set(String key, Object value, Long time, TimeUnit unit) {
        stringRedisTemplate.opsForValue().set(key, JSONUtil.toJsonStr(value), time, unit);
    }

    /**
     * 缓存空值解决缓存穿透
     * @param keyPrefix redis key前缀
     * @param id 查询条件的数据，暂定为id
     * @param type 返回值的类型
     * @param dbFallback 调用mybatis-plus函数
     * @param time 缓存存在时间
     * @param unit 时间单位
     * @param <R> 泛型——返回值类型
     * @param <ID> 泛型——查询条件数据类型
     * @return
     */
    public <R,ID> R queryWithPassThrough(
            String keyPrefix, ID id, Class<R> type, Function<ID, R> dbFallback, Long time, TimeUnit unit){
        // key由 前缀 + id 组成
        String key =keyPrefix + id;
        String json = stringRedisTemplate.opsForValue().get(key);
        //判断redis内是否有缓存
        if(StrUtil.isNotBlank(json)){
            return JSONUtil.toBean(json, type);
        }
        //缓存命中却无内容
        if(json != null){
            return null;
        }

        R r = dbFallback.apply(id);
        //若数据库不存在，存入空值并设置过期时间
        if(r == null){
            stringRedisTemplate.opsForValue().set(key, "", CACHE_NULL_TTL, TimeUnit.MINUTES);
            return null;
        }
        this.set(key, r, time, unit);
        return r;
    }



}
